// Copyright 2015 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// An integration test service that covers all the method signature permutations
// of unary/streaming requests/responses.
syntax = "proto3";

import "grpc/testing/control.proto";

package grpc.testing;

option java_multiple_files = true;
option java_package = "io.grpc.testing";
option java_outer_classname = "ReportQpsScenarioServiceProto";

service ReportQpsScenarioService {
  // Report results of a QPS test benchmark scenario.
  rpc ReportScenario(ScenarioResult) returns (Void);
}
